@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2023, GrammarSoft ApS
+* Copyright (C) 2007-2025, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -398,7 +398,7 @@ void ApertiumApplicator::runGrammarOnText(std::istream& input, std::ostream& out
 
 			blank.append(u"\"<");
 			UChar* p = &token[1];
-			for (; *p && *p != '/' && *p != '<'; ++p) {
+			for (; *p && *p != '/' && *p != '<' && *p != '$'; ++p) {
 				if (*p == '\\') {
 					++p;
 				}
@@ -406,6 +406,7 @@ void ApertiumApplicator::runGrammarOnText(std::istream& input, std::ostream& out
 			}
 			blank.append(u">\"");
 			cCohort->wordform = addTag(blank);
+			blank.clear();
 
 			// Handle the static reading of ^estació<n><f><sg>/season<n><sg>/station<n><sg>$
 			// Gobble up all <tags> until the first / or $ and stuff them in the static reading
@@ -807,10 +808,6 @@ void ApertiumApplicator::printReading(Reading* reading, std::ostream& output, Ap
 }
 
 void ApertiumApplicator::printReading(Reading* reading, std::ostream& output) {
-	if (reading->noprint) {
-		return;
-	}
-
 	size_t firstlower = 0;
 	ApertiumCasing casing = ApertiumCasing::Lower;
 
@@ -848,7 +845,7 @@ void ApertiumApplicator::printReading(Reading* reading, std::ostream& output) {
 						break;
 					}
 					else {
-						uppercaseseen++;
+						++uppercaseseen;
 					}
 				}
 			}
@@ -868,6 +865,13 @@ void ApertiumApplicator::printReading(Reading* reading, std::ostream& output) {
 }
 
 void ApertiumApplicator::printCohort(Cohort* cohort, std::ostream& output, bool profiling) {
+	if (cohort->local_number == 0 || (cohort->type & CT_REMOVED)) {
+		if (!cohort->text.empty()) {
+			u_fprintf(output, "%S", cohort->text.data());
+		}
+		return;
+	}
+
 	if (!profiling) {
 		cohort->unignoreAll();
 
@@ -918,6 +922,9 @@ void ApertiumApplicator::printCohort(Cohort* cohort, std::ostream& output, bool
 	//Tag::printTagRaw(output, grammar->single_tags[cohort->wordform]);
 	std::sort(cohort->readings.begin(), cohort->readings.end(), Reading::cmp_number);
 	for (auto reading : cohort->readings) {
+		if (reading->noprint) {
+			continue;
+		}
 		if (need_slash) {
 			u_fprintf(output, "/");
 		}
@@ -934,6 +941,9 @@ void ApertiumApplicator::printCohort(Cohort* cohort, std::ostream& output, bool
 	if (trace) {
 		std::sort(cohort->delayed.begin(), cohort->delayed.end(), Reading::cmp_number);
 		for (auto reading : cohort->delayed) {
+			if (reading->noprint) {
+				continue;
+			}
 			if (need_slash) {
 				u_fprintf(output, "/%C", not_sign);
 			}
@@ -945,6 +955,9 @@ void ApertiumApplicator::printCohort(Cohort* cohort, std::ostream& output, bool
 		}
 		std::sort(cohort->deleted.begin(), cohort->deleted.end(), Reading::cmp_number);
 		for (auto reading : cohort->deleted) {
+			if (reading->noprint) {
+				continue;
+			}
 			if (need_slash) {
 				u_fprintf(output, "/%C", not_sign);
 			}
@@ -964,11 +977,6 @@ void ApertiumApplicator::printCohort(Cohort* cohort, std::ostream& output, bool
 	if (!cohort->text.empty()) {
 		u_fprintf(output, "%S", cohort->text.data());
 	}
-	for (auto& c : cohort->removed) {
-		if (!c->text.empty()) {
-			u_fprintf(output, "%S", c->text.data());
-		}
-	}
 }
 
 void ApertiumApplicator::printSingleWindow(SingleWindow* window, std::ostream& output, bool profiling) {
@@ -977,18 +985,7 @@ void ApertiumApplicator::printSingleWindow(SingleWindow* window, std::ostream& o
 		u_fprintf(output, "%S", window->text.data());
 	}
 
-	for (uint32_t c = 0; c < window->cohorts.size(); c++) {
-		Cohort* cohort = window->cohorts[c];
-
-		if (c == 0) { // Skip magic cohort
-			for (auto& c : cohort->removed) {
-				if (!c->text.empty()) {
-					u_fprintf(output, "%S", c->text.data());
-				}
-			}
-			continue;
-		}
-
+	for (auto& cohort : window->all_cohorts) {
 		printCohort(cohort, output, profiling);
 		u_fflush(output);
 	}
