@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2023, GrammarSoft ApS
+* Copyright (C) 2007-2025, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -439,7 +439,7 @@ void GrammarApplicator::printReading(const Reading* reading, std::ostream& outpu
 		}
 	}
 
-	if (reading->parent->type & CT_RELATED) {
+	if (print_ids || reading->parent->type & CT_RELATED) {
 		u_fprintf(output, " ID:%u", reading->parent->global_number);
 		if (!reading->parent->relations.empty()) {
 			for (const auto& miter : reading->parent->relations) {
@@ -478,6 +478,13 @@ void GrammarApplicator::printCohort(Cohort* cohort, std::ostream& output, bool p
 		u_fprintf(output, "# RULE TARGET BEGIN\n");
 	}
 
+	if (!cohort->wblank.empty()) {
+		u_fprintf(output, "%S", cohort->wblank.data());
+		if (!ISNL(cohort->wblank.back())) {
+			u_fputc('\n', output);
+		}
+	}
+
 	if (cohort->type & CT_REMOVED) {
 		if (!trace || trace_no_removed) {
 			goto removed;
@@ -528,10 +535,6 @@ removed:
 		}
 	}
 
-	for (auto iter : cohort->removed) {
-		printCohort(iter, output, profiling);
-	}
-
 	if (profiling && cohort == rule_target) {
 		u_fprintf(output, "# RULE TARGET END\n");
 	}
@@ -562,9 +565,7 @@ void GrammarApplicator::printSingleWindow(SingleWindow* window, std::ostream& ou
 		}
 	}
 
-	uint32_t cs = UI32(window->cohorts.size());
-	for (uint32_t c = 0; c < cs; c++) {
-		Cohort* cohort = window->cohorts[c];
+	for (auto& cohort : window->all_cohorts) {
 		printCohort(cohort, output, profiling);
 	}
 
