@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2023, GrammarSoft ApS
+* Copyright (C) 2007-2025, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -425,6 +425,13 @@ void NicelineApplicator::printCohort(Cohort* cohort, std::ostream& output, bool
 		goto removed;
 	}
 
+	if (!cohort->wblank.empty()) {
+		u_fprintf(output, "%S", cohort->wblank.data());
+		if (!ISNL(cohort->wblank.back())) {
+			u_fputc('\n', output);
+		}
+	}
+
 	u_fprintf(output, "%.*S", cohort->wordform->tag.size() - 4, cohort->wordform->tag.data() + 2);
 	if (cohort->wread && !did_warn_statictags) {
 		u_fprintf(ux_stderr, "Warning: Niceline CG format cannot output static tags! You are losing information!\n");
@@ -465,9 +472,7 @@ void NicelineApplicator::printSingleWindow(SingleWindow* window, std::ostream& o
 		}
 	}
 
-	uint32_t cs = UI32(window->cohorts.size());
-	for (uint32_t c = 0; c < cs; c++) {
-		Cohort* cohort = window->cohorts[c];
+	for (auto& cohort : window->all_cohorts) {
 		printCohort(cohort, output, profiling);
 	}
 
