@@ -1,22 +1,20 @@
 /*
-* Copyright (C) 2007-2018, GrammarSoft ApS
+* Copyright (C) 2007-2025, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
-* This file is part of VISL CG-3
-*
-* VISL CG-3 is free software: you can redistribute it and/or modify
+* This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
-* VISL CG-3 is distributed in the hope that it will be useful,
+* This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with VISL CG-3.  If not, see <http://www.gnu.org/licenses/>.
+* along with this progam.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #pragma once
@@ -94,6 +92,7 @@ void cg3_applicator_setoption(cg3_applicator* applicator, cg3_option option, voi
 void cg3_applicator_free(cg3_applicator* applicator);
 
 void cg3_run_grammar_on_text(cg3_applicator*, std_istream*, std_ostream*);
+void cg3_run_grammar_on_text_fns(cg3_applicator*, const char* input, const char* output);
 
 cg3_mwesplitapplicator* cg3_mwesplitapplicator_create();
 #define cg3_mwesplitapplicator_free cg3_applicator_free
