@@ -1,22 +1,19 @@
 /*
-* Copyright (C) 2007-2021, GrammarSoft ApS
+* Copyright (C) 2007-2025, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
-* Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
-* This file is part of VISL CG-3
-*
-* VISL CG-3 is free software: you can redistribute it and/or modify
+* This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
-* VISL CG-3 is distributed in the hope that it will be useful,
+* This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with VISL CG-3.  If not, see <http://www.gnu.org/licenses/>.
+* along with this progam.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #pragma once
@@ -34,7 +31,7 @@ inline const char *basename(const char *path) {
 		const char *pos = std::max(strrchr(path, '\\'), strrchr(path, '/'));
 
 		if (pos != nullptr) { // If a \ char was found...
-			if (*(pos + 1) != 0) { // If it is not the last character in the string...
+			if (pos[1] != 0) { // If it is not the last character in the string...
 				return pos + 1; // then return a pointer to the first character after \.
 			}
 			return pos; // else return a pointer to \.
@@ -85,8 +82,6 @@ inline bool ux_stripBOM(std::istream& stream) {
 	return true;
 }
 
-size_t get_line_clean(CG3::UString& line, CG3::UString& cleaned, std::istream& input, bool keep_tabs = false);
-
 // ICU std::istream input wrappers
 UChar* u_fgets(UChar* s, int32_t n, std::istream& input);
 
@@ -171,7 +166,7 @@ inline bool ux_simplecasecmp(const UChar* a, const UChar* b, const size_t n) {
 
 	// If there is a combining character after the last plain letter, it's not a match
 	// But since that's a whole function call, short circuit for most likely suffixes
-	return a[n] == 0 || ISSPACE(a[n]) || u_getCombiningClass(a[n]) == 0;
+	return a[n] == 0 || ISSPACE(a[n]) || ISDELIM(a[n]) || u_getCombiningClass(a[n]) == 0;
 }
 
 inline bool ux_simplecasecmp(const UChar* a, const UString& b) {
@@ -182,6 +177,10 @@ inline bool ux_simplecasecmp(const UChar* a, const UStringView& b) {
 	return ux_simplecasecmp(a, b.data(), b.size());
 }
 
+inline bool ux_simplecasecmp(const UStringView& a, const UStringView& b) {
+	return ux_simplecasecmp(a.data(), b.data(), b.size());
+}
+
 inline bool ux_strCaseCompare(const UString& a, const UString& b) {
 	UErrorCode status = U_ZERO_ERROR;
 	auto rv = u_strCaseCompare(a.data(), SI32(a.size()), b.data(), SI32(b.size()), U_FOLD_CASE_DEFAULT, &status);
@@ -241,7 +240,9 @@ inline UChar* ux_bufcpy(UChar* dst, const UChar* src, size_t n) {
 }
 
 std::string ux_dirname(const char* in);
-void findAndReplace(UnicodeString& str, CG3::UStringView from, CG3::UStringView to);
+void findAndReplace(UnicodeString& str, UStringView from, UStringView to);
+size_t get_line_clean(UString& line, UString& cleaned, std::istream& input, bool keep_tabs = false);
+
 }
 
 #endif
